import os
from .githelper import get_tag, get_project_name_from_url
from .config_data import load_config
from codegenhelper import debug
import demjson

def init_tag_for_project(giturl, tag, config_name, location=os.getcwd()):
    get_tag(giturl, tag, location)

    def load_tag(dep_config):
        get_tag(dep_config["giturl"], dep_config["tag"], location)
        
        return demjson.decode_file(os.path.join(location, get_project_name_from_url(dep_config["giturl"]), "deploy", dep_config["config_name"] + ".cfg"))
    
    load_config(os.path.join(location, debug(get_project_name_from_url(giturl), "url_project_nameg"), "deploy", config_name + ".cfg"), load_tag)



import os
from .defaults import write as write_defaults, override_values, all as all_defaults
from .roles import build as roles_build, link as roles_link, load as roles_load, link_src_to_deploy
from .debug import simple as debug_simple
import yaml
from .tools.file import get_file_name, get_file_only_name, put_file
from .tools.folder import put_folder
import demjson
from fn import _, F, iters
from .tools.dicthelper import update_dict
from .util import get_config_path, get_deploy_path, get_deploy_host
from .tools.data_convert import dict2assignments
import json

def run_deploy(configuration_path):
    '''configuration_path: it is the folder path which contains the main.sh to launch the deploy script'''
    import subprocess
    
    def run(current_path):
        try:
            os.chdir(configuration_path)
            p = subprocess.Popen("bash main.sh", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            while p.poll() is None:
                line = p.stdout.readline()
                if line:
                    print(line.strip())
        finally:
            os.chdir(current_path)

    run(os.getcwd())


def build_deploy_script(project_name, config_name, only_structure=False):
    def yml_file_folder(): return put_folder(os.path.abspath(get_deploy_path(project_name, config_name)))
    def yml_file_path(): return os.path.join(yml_file_folder(), "main.yml")
    def host_file(): return os.path.join(yml_file_folder(), debug_simple(get_file_only_name(yml_file_path()), "host file name") + ".host")
    def bash_file(): return os.path.join(yml_file_folder(), get_file_only_name(yml_file_path()) + ".sh")
    def get_roles_data():return roles_load(debug_simple(get_config_path(project_name, config_name), "config_path"))
    def write_playbook(content): open(put_file(yml_file_path()), 'w').write(content)

    # write data to file
    write_playbook(roles_build(get_roles_data()))
    # build link for role folders
    roles_link(get_roles_data(), \
               debug_simple(yml_file_folder(), "link_root") \
    )

    # link src folder to deploy/roles/main/files/src for deployment
    link_src_to_deploy(get_roles_data())

    def default_vals(): return all_defaults(yml_file_folder(), [role["name"] for role in get_roles_data()])

    # build inventory files on roles
    write_defaults(host_file(), default_vals())

    # write the defaults to cfg file
    (F(lambda config_path: demjson.decode_file(config_path)) >> \
        F(lambda new_vals, old_json: update_dict(old_json, new_vals), {} if only_structure else {"predefined_variables": default_vals()}) >> \
        F(lambda json_data: json.dumps(json_data, indent=4, ensure_ascii=False)) >> \
        F(lambda content: open(get_config_path(project_name, config_name), 'w').write(content)))(get_config_path(project_name, config_name))


    # build the bash file
    open(bash_file(), "w") \
        .write("sudo ansible-playbook ./{yml_file} -i ./{host_file}" \
               .format(yml_file = get_file_name(put_file(yml_file_path())), \
                       host_file = get_file_name(host_file()) \
               ) \
        )



def deploy_config(project_name, config_name):
    def get_banyan_config():
        return demjson.decode_file(get_config_path(project_name, config_name))
    def get_predefined_variables(banyan_config):
        return banyan_config["predefined_variables"] if "predefined_variables" in banyan_config else {}
    def get_existing_items_from_host():
        return (F(filter, _.call("find", "=") != -1) \
            >> F(map, lambda item: (item.split("=")[0], item.split("=")[1].strip())) \
            >> F(dict))(open(get_deploy_host(project_name, config_name), 'r').readlines())
    def save_items_to_host(lines):
        open(get_deploy_host(project_name, config_name), 'w').write("[all:vars]" + os.linesep + lines)

    (F(get_banyan_config) >> \
        F(get_predefined_variables) >> \
        F(update_dict, get_existing_items_from_host()) >> \
        F(dict2assignments) >> \
        F(lambda items: os.linesep.join(items)) >> \
        F(save_items_to_host))()

    





import os
import demjson
from .assert_util import not_none as assert_not_none
from .debug import simple as debug_simple
import logging
from .util import get_project_path, extract_project_path, get_config_path
from pathlib import Path
logger = logging.getLogger(__name__)
from fn import F

def load(config_path):
    '''
config_path: the path of the banyan configuration file
'''
    def get_cfg(dep_json):
        def load_cfg(cfg_path):
            return demjson.decode_file(cfg_path)

        return (F(get_config_path) >> F(load_cfg))(dep_json["project_name"], dep_json["config_name"] if "config_name" in dep_json else None)

    def get_roles(cfg_json, project_path):
        return [{"name": "%s_%s" % (debug_simple(os.path.split(project_path), "get_roles path")[1], role_name), \
                 "local": project_path,
                 "original_name": role_name} for role_name in cfg_json["roles_seq"]] if "roles_seq" in debug_simple(cfg_json, "get_roles cf_json") else []
    
    def handle(cfg_json, depended_roles, project_path):
        debug_simple(project_path, "handle_project_path")
        try:
            return [y for x in [handle(debug_simple(get_cfg(dep), "json in recursive"), depended_roles, get_project_path(dep["project_name"])) for dep in cfg_json["dependencies"]] for y in x] + get_roles(debug_simple(cfg_json, "get_roles_json_recursive"), debug_simple(project_path, "get_roles_path_recursive")) \
            if "dependencies" in cfg_json \
               else depended_roles + get_roles(debug_simple(cfg_json, "no dep json"), debug_simple(project_path, "no dep path"))
        except Exception:
            logger.debug("error happen in handle with project_path:%s\ncfg_json:%s", project_path, cfg_json)
            raise

    return handle(demjson.decode_file(config_path), [], debug_simple(extract_project_path(config_path)))


def build(roles):
    from jinja2 import Template
    '''
build the roles into the entry yaml file
roles: a list for role. the sturcture of role can be reference in get_roles
'''
    return Template('''
---

- name: deploy
  hosts: localhost
  become: true
  become_method: sudo
  roles:
{% for role in objs %}    - {{role}}
{% endfor %}
...
  
''').render(objs = [role["name"] for role in roles])


def link(roles, path):
    '''
link the roles
'''
    def get_src(role):
        return os.path.join(role["local"], "deploy", "roles", role["original_name"])

    def create_folder(name, parent_path):
        def create(path):
            if os.path.exists(path):
                return path
            os.makedirs(path)
            return path
        return create(os.path.join(parent_path, name))
    
    def get_dest(role):
        return os.path.join(create_folder("roles", os.path.abspath(path)) , role["name"])
    
    def link_role(role):
        if not os.path.exists(get_dest(role)):
            os.symlink(os.path.abspath(debug_simple(get_src(role), "src")), debug_simple(get_dest(role), "dest"))
        
    [link_role(role) for role in roles]


def link_src_to_deploy(rolesdata):
    ''' link src folder to deploy/roles/main/files/src for deployment '''
    
    def link(project_folder, src_name, link_folder):
        debug_simple(project_folder, 'project_folder')
        def put(target_path):
            if not os.path.exists(target_path):
                os.makedirs(target_path)
            return target_path
        
        def link_path(source_path, target_link_path):
            if os.path.exists(source_path) and not Path(target_link_path).is_symlink():
                os.symlink(source_path, target_link_path)

        link_path(os.path.abspath(os.path.join(project_folder, src_name)),  os.path.join(\
                                                                                         put(os.path.abspath(os.path.join(project_folder, link_folder))),\
                                                                                         src_name)\
        )
        
    [link(project_folder, "src", "deploy/roles/main/files") for project_folder in list(set([p["local"] for p in rolesdata]))]
        
